@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #--------------------------------------------------------------------------------
-#www2sms.py v1.2.1
+#www2sms.py v1.3.0
 #Copyright Bjoern Olausson
 #--------------------------------------------------------------------------------
 #This program is free software; you can redistribute it and/or modify
@@ -22,20 +22,27 @@
 #--------------------------------------------------------------------------------
 #--------------------------------------------------------------------------------
 #
-
-import os, errno, pwd, sys, math, sqlite3, time, calendar, re, socket
-import ctypes, uuid
+import os, errno, pwd, sys, math, sqlite3, time, calendar, re, socket, tarfile
+import ctypes, uuid, subprocess
 from PyQt4.QtGui import QApplication
 from PyQt4 import QtCore
 from PyQt4.QtGui import QMainWindow, QDialog, QPushButton, QMessageBox, QPixmap
-import www2sms_providers
 from www2sms_gui_main import Ui_MainWindow
 from www2sms_gui_widget_account import Ui_FormAccount
 from www2sms_gui_dialog_progress import Ui_Form_Progress
+__version__ = '1.3.0'
+__version_info__ = tuple([ int(num) for num in __version__.split('.')])
 
-VERSION = "v1.2.1"
 USER = pwd.getpwuid(os.getuid())[0]
 CONFIG_DIR = "/home/%s/MyDocs/www2sms" %(USER)
+GPG_DIR = "%s/gnupg" %(CONFIG_DIR)
+SCRIPT = os.path.abspath(__file__)
+sys.path.append(CONFIG_DIR)
+
+try:
+	import www2sms_providers_web as www2sms_providers
+except:
+	import www2sms_providers
 
 #Get the list of providers based on the classes implemented.
 list_providers = []
@@ -47,6 +54,10 @@ for label in dir(www2sms_providers):
 	list_providers.append(provider.getName())
 	providers.append(provider)
 
+def restart_program():
+    python = sys.executable
+    os.execl(python, python, SCRIPT)
+
 class MainWindow(QMainWindow, Ui_MainWindow):
 	def __init__(self, win_parent = None):
 		#Init the base class
@@ -55,9 +66,11 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 		self.ui.setupUi(self)
 		# Setup GUI elements
 		self.ui.label_www2sms.setText('www2sms')
-		self.ui.label_www2sms_version.setText(VERSION)
+		self.ui.label_www2sms_version.setText("v%s" %(__version__))
 		pixmap = QPixmap('/usr/share/pixmaps/www2sms.png')
 		self.ui.label_www2sms_logo.setPixmap(pixmap)
+		self.ui.label_providers.setText('providers')
+		self.ui.label_providers_version.setText("v%s" %(www2sms_providers.__version__))
 		#The Account Form
 		class FormAccount(QDialog, Ui_FormAccount):
 			def __init__(self, win_parent = None):
@@ -111,8 +124,82 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 		self.cpsms(provider_index)
 		QtCore.QObject.connect(self.ui.actionAdv_Provider_Settings, QtCore.SIGNAL("triggered(bool)"), self.ShowDialogAdv)
 		QtCore.QObject.connect(self.ui.actionAccount_Setup, QtCore.SIGNAL("triggered(bool)"), self.ShowDialogAccSet)
+		QtCore.QObject.connect(self.ui.actionUpdate, QtCore.SIGNAL("triggered(bool)"), self.update)
 		self.cpsms(provider_index)
-    
+
+	def update(self):
+		from urllib2 import urlopen as uo
+		from BeautifulSoup import BeautifulSoup as bs
+		cur_version = www2sms_providers.__version__
+		cur_version_info = www2sms_providers.__version_info__
+		try:
+			page = uo("http://olausson.de/downloads/view.download/2/2")
+		except:
+			QMessageBox.warning(self, 'www2sms', 'Failed to retrieve version information')
+			return
+		soup = bs(page)
+		#Get current version of www2sms_providers.py
+		new_version = soup.findAll(text=re.compile(r'Version:[0-9]*.[0-9]*.[0-9]*'))[0].split(":")[1]
+		new_version_info = tuple([ int(num) for num in new_version.split('.')])
+		if new_version_info > cur_version_info:
+			try:
+			    os.makedirs(GPG_DIR, 0700)
+			except OSError, e:
+			    # Pass if the directory exist, else raise a exception
+			    if e.errno == errno.EEXIST:
+				pass
+			    else:
+				raise Exception("Failed to create %s. (%s, %s)" %(GPG_DIR, errno, strerror))
+			URL = "http://olausson.de/downloads/finish/2/2"
+			try:
+				dl = uo(URL)
+			except:
+				QMessageBox.warning(self, 'www2sms', 'Failed to download new version')
+				return
+			f = open(CONFIG_DIR+"/www2sms_providers.py.tar.gpg", 'w')
+			f.write(dl.read())
+			f.close()
+			# Verify key against my pubkey fingerprint
+			MyFP = "1707 153E 7837 DAE2 7B3D  4C90 7716 1766 3CB1 2D4B"
+			MyID = "3CB12D4B"
+			# check if we need to import my pubkey:
+			cmd = ["gpg", "--no-permission-warning", "--preserve-permissions", "--lock-never", "--homedir", GPG_DIR, "--list-keys"]
+			ps = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+			stdo, stde = ps.communicate()
+			if not re.search(MyID, stdo):
+				# import my pubkey:
+				cmd = ["gpg", "--no-permission-warning", "--preserve-permissions", "--lock-never", "--homedir", GPG_DIR, "--keyserver", "pgpkeys.pca.dfn.de", "--recv-key", MyID]
+				ps = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+				stdo, stde = ps.communicate()
+				if ps.returncode != 0:
+					QMessageBox.warning(self, 'www2sms', 'Failed to import GPG signature')
+					return
+			# verify and decrypt www2sms_providers.py.tar.gpg
+			cmd = ["gpg", "--no-permission-warning", "--preserve-permissions", "--lock-never", "--homedir", GPG_DIR, "-q", "--yes", CONFIG_DIR+"/www2sms_providers.py.tar.gpg"]
+			ps = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+			stdo, stde = ps.communicate()
+			# Check if the file has a valid signature and if the fingerprint of the signature matches my fingerprint
+			if ps.returncode != 0 or not re.search(MyFP, stde):
+				QMessageBox.warning(self, 'www2sms', 'www2sms_providers.tar.gpg has an invalid signature and might have been compromised.\nPlease notify the author (maemo@olausson.de')
+				return
+			#extract the tar file
+			try:
+				tar = tarfile.open(CONFIG_DIR+"/www2sms_providers.py.tar")
+				tar.extractall()
+				tar.close()
+			except:
+				QMessageBox.warning(self, 'www2sms', 'Failed to unpack the tar archive')
+				return
+			finally:
+				os.remove(CONFIG_DIR+"/www2sms_providers.py.tar")
+				os.remove(CONFIG_DIR+"/www2sms_providers.py.tar.gpg")
+			os.rename(CONFIG_DIR+"/www2sms_providers.py", CONFIG_DIR+"/www2sms_providers_web.py")
+			reply = QMessageBox.question(self, 'www2sms', 'Providers updated from %s to %s\nRestart now?' %(cur_version, new_version), QMessageBox.Yes, QMessageBox.No)
+		    	if reply == QMessageBox.Yes:
+				restart_program()
+		else:
+			QMessageBox.information(self, 'www2sms', "No update available at the moment (%s)" %(cur_version))
+
 	def enableTabs(self, provider_index):
 		## Enable the SMS SEND Button if we have an account, otherwise not
 		if self.AccSet.ui.comboBox_SetupAlias.currentText():
