@@ -1,5 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+__version__ = '1.0.0'
+__version_info__ = tuple([ int(num) for num in __version__.split('.')])
 
 import sys, re, mechanize, base64, urllib2, time, calendar, urllib, sqlite3
 from lxml.html import soupparser
@@ -7,10 +9,7 @@ from PyQt4 import QtCore, QtGui
 from PyQt4.QtGui import QApplication, QDialog, QImage, QPixmap
 from www2sms_gui_dialog_captcha import Ui_DialogCaptcha
 
-def UpdatePbar(self, MainWindow, app, value, label):
-	MainWindow.Stat.ui.label.setText(label)
-	MainWindow.Stat.ui.progressBar.setProperty("value", value)
-	app.processEvents()
+
 
 class ShowCAPTCHA(QDialog, Ui_DialogCaptcha):
 	def __init__(self, win_parent = None):
@@ -19,6 +18,11 @@ class ShowCAPTCHA(QDialog, Ui_DialogCaptcha):
 		self.ui = Ui_DialogCaptcha()
 		self.ui.setupUi(self)
 
+def UpdatePbar(self, MainWindow, app, value, label):
+	MainWindow.Stat.ui.label.setText(label)
+	MainWindow.Stat.ui.progressBar.setProperty("value", value)
+	app.processEvents()
+
 def SearchCAPTCHA(self, baseurl, response, regex, element, attrib):
 	def gen_elements(tag, root):
 		if root.tag == tag:
@@ -326,10 +330,12 @@ class provider_o2online_de(provider):
 			#------------------------------------
 			# Check for a redirect
 			#------------------------------------
-			for link in br.links(text_regex="SMS"):
-				print link
 			if re.search("weitergeleitet", response):
-				return error(self, MainWindow, 'Redirection failed', 'Step 5: Redirection failed, please try again')
+				try:
+					br.select_form(nr = 0)
+					br.submit(nr=0)
+				except Exception, e:
+					return error(self, MainWindow, e, 'Step 5: Redirection failed, please try again')
 
 			try:
 				br.find_link(text_regex="Mail")
